@@ -16,9 +16,9 @@ from pandas.testing import assert_frame_equal  # noqa: F401
 
 import xarray.testing
 from xarray import Dataset
-from xarray.core import utils
+from xarray.coding.times import _STANDARD_CALENDARS as _STANDARD_CALENDARS_UNSORTED
 from xarray.core.duck_array_ops import allclose_or_equiv  # noqa: F401
-from xarray.core.indexing import ExplicitlyIndexed
+from xarray.core.extension_array import PandasExtensionArray
 from xarray.core.options import set_options
 from xarray.core.variable import IndexVariable
 from xarray.testing import (  # noqa: F401
@@ -26,6 +26,14 @@ from xarray.testing import (  # noqa: F401
     assert_duckarray_allclose,
     assert_duckarray_equal,
 )
+from xarray.tests.arrays import (  # noqa: F401
+    ConcatenatableArray,
+    DuckArrayWrapper,
+    FirstElementAccessibleArray,
+    InaccessibleArray,
+    IndexableArray,
+    UnexpectedDataAccess,
+)
 
 # import mpl and change the backend before other mpl imports
 try:
@@ -41,6 +49,7 @@ except ImportError:
 warnings.filterwarnings("ignore", "'urllib3.contrib.pyopenssl' module is deprecated")
 warnings.filterwarnings("ignore", "Deprecated call to `pkg_resources.declare_namespace")
 warnings.filterwarnings("ignore", "pkg_resources is deprecated as an API")
+warnings.filterwarnings("ignore", message="numpy.ndarray size changed")
 
 arm_xfail = pytest.mark.xfail(
     platform.machine() == "aarch64" or "arm" in platform.machine(),
@@ -52,7 +61,11 @@ def assert_writeable(ds):
     readonly = [
         name
         for name, var in ds.variables.items()
-        if not isinstance(var, IndexVariable) and not var.data.flags.writeable
+        if not isinstance(var, IndexVariable)
+        and not isinstance(
+            var.data, PandasExtensionArray | pd.api.extensions.ExtensionArray
+        )
+        and not var.data.flags.writeable
     ]
     assert not readonly, readonly
 
@@ -79,6 +92,7 @@ def _importorskip(
 
 has_matplotlib, requires_matplotlib = _importorskip("matplotlib")
 has_scipy, requires_scipy = _importorskip("scipy")
+has_scipy_ge_1_13, requires_scipy_ge_1_13 = _importorskip("scipy", "1.13")
 with warnings.catch_warnings():
     warnings.filterwarnings(
         "ignore",
@@ -96,22 +110,52 @@ with warnings.catch_warnings():
     )
 
     has_h5netcdf, requires_h5netcdf = _importorskip("h5netcdf")
-has_pynio, requires_pynio = _importorskip("Nio")
 has_cftime, requires_cftime = _importorskip("cftime")
 has_dask, requires_dask = _importorskip("dask")
-with warnings.catch_warnings():
-    warnings.filterwarnings(
-        "ignore",
-        message="The current Dask DataFrame implementation is deprecated.",
-        category=DeprecationWarning,
-    )
-    has_dask_expr, requires_dask_expr = _importorskip("dask_expr")
+has_dask_ge_2024_08_1, requires_dask_ge_2024_08_1 = _importorskip(
+    "dask", minversion="2024.08.1"
+)
+has_dask_ge_2024_11_0, requires_dask_ge_2024_11_0 = _importorskip("dask", "2024.11.0")
+has_dask_ge_2025_1_0, requires_dask_ge_2025_1_0 = _importorskip("dask", "2025.1.0")
+if has_dask_ge_2025_1_0:
+    has_dask_expr = True
+    requires_dask_expr = pytest.mark.skipif(not has_dask_expr, reason="should not skip")
+else:
+    with warnings.catch_warnings():
+        warnings.filterwarnings(
+            "ignore",
+            message="The current Dask DataFrame implementation is deprecated.",
+            category=DeprecationWarning,
+        )
+        has_dask_expr, requires_dask_expr = _importorskip("dask_expr")
 has_bottleneck, requires_bottleneck = _importorskip("bottleneck")
 has_rasterio, requires_rasterio = _importorskip("rasterio")
 has_zarr, requires_zarr = _importorskip("zarr")
+has_zarr_v3, requires_zarr_v3 = _importorskip("zarr", "3.0.0")
+has_zarr_v3_dtypes, requires_zarr_v3_dtypes = _importorskip("zarr", "3.1.0")
+has_zarr_v3_async_oindex, requires_zarr_v3_async_oindex = _importorskip("zarr", "3.1.2")
+if has_zarr_v3:
+    import zarr
+
+    # manual update by checking attrs for now
+    # TODO: use version specifier
+    # installing from git main is giving me a lower version than the
+    # most recently released zarr
+    has_zarr_v3_dtypes = hasattr(zarr.core, "dtype")
+    has_zarr_v3_async_oindex = hasattr(zarr.AsyncArray, "oindex")
+
+    requires_zarr_v3_dtypes = pytest.mark.skipif(
+        not has_zarr_v3_dtypes, reason="requires zarr>3.1.0"
+    )
+    requires_zarr_v3_async_oindex = pytest.mark.skipif(
+        not has_zarr_v3_async_oindex, reason="requires zarr>3.1.1"
+    )
+
+
 has_fsspec, requires_fsspec = _importorskip("fsspec")
 has_iris, requires_iris = _importorskip("iris")
-has_numbagg, requires_numbagg = _importorskip("numbagg", "0.4.0")
+has_numbagg, requires_numbagg = _importorskip("numbagg")
+has_pyarrow, requires_pyarrow = _importorskip("pyarrow")
 with warnings.catch_warnings():
     warnings.filterwarnings(
         "ignore",
@@ -126,7 +170,9 @@ has_cartopy, requires_cartopy = _importorskip("cartopy")
 has_pint, requires_pint = _importorskip("pint")
 has_numexpr, requires_numexpr = _importorskip("numexpr")
 has_flox, requires_flox = _importorskip("flox")
-has_pandas_ge_2_2, __ = _importorskip("pandas", "2.2")
+has_netcdf, requires_netcdf = _importorskip("netcdf")
+has_pandas_ge_2_2, requires_pandas_ge_2_2 = _importorskip("pandas", "2.2")
+has_pandas_3, requires_pandas_3 = _importorskip("pandas", "3.0.0")
 
 
 # some special cases
@@ -134,22 +180,64 @@ has_scipy_or_netCDF4 = has_scipy or has_netCDF4
 requires_scipy_or_netCDF4 = pytest.mark.skipif(
     not has_scipy_or_netCDF4, reason="requires scipy or netCDF4"
 )
+has_h5netcdf_or_netCDF4 = has_h5netcdf or has_netCDF4
+requires_h5netcdf_or_netCDF4 = pytest.mark.skipif(
+    not has_h5netcdf_or_netCDF4, reason="requires h5netcdf or netCDF4"
+)
 has_numbagg_or_bottleneck = has_numbagg or has_bottleneck
 requires_numbagg_or_bottleneck = pytest.mark.skipif(
-    not has_scipy_or_netCDF4, reason="requires scipy or netCDF4"
+    not has_numbagg_or_bottleneck, reason="requires numbagg or bottleneck"
 )
-# _importorskip does not work for development versions
-has_pandas_version_two = Version(pd.__version__).major >= 2
-requires_pandas_version_two = pytest.mark.skipif(
-    not has_pandas_version_two, reason="requires pandas 2.0.0"
-)
-has_numpy_array_api, requires_numpy_array_api = _importorskip("numpy", "1.26.0")
-has_h5netcdf_ros3, requires_h5netcdf_ros3 = _importorskip("h5netcdf", "1.3.0")
+has_numpy_2, requires_numpy_2 = _importorskip("numpy", "2.0.0")
+has_flox_0_9_12, requires_flox_0_9_12 = _importorskip("flox", "0.9.12")
 
+has_array_api_strict, requires_array_api_strict = _importorskip("array_api_strict")
+
+parametrize_zarr_format = pytest.mark.parametrize(
+    "zarr_format",
+    [
+        pytest.param(2, id="zarr_format=2"),
+        pytest.param(
+            3,
+            marks=pytest.mark.skipif(
+                not has_zarr_v3,
+                reason="zarr-python v2 cannot understand the zarr v3 format",
+            ),
+            id="zarr_format=3",
+        ),
+    ],
+)
+
+
+def _importorskip_h5netcdf_ros3(has_h5netcdf: bool):
+    if not has_h5netcdf:
+        return has_h5netcdf, pytest.mark.skipif(
+            not has_h5netcdf, reason="requires h5netcdf"
+        )
+
+    import h5py
+
+    h5py_with_ros3 = h5py.get_config().ros3
+
+    return h5py_with_ros3, pytest.mark.skipif(
+        not h5py_with_ros3,
+        reason="requires h5netcdf>=1.3.0 and h5py with ros3 support",
+    )
+
+
+has_h5netcdf_ros3, requires_h5netcdf_ros3 = _importorskip_h5netcdf_ros3(has_h5netcdf)
 has_netCDF4_1_6_2_or_above, requires_netCDF4_1_6_2_or_above = _importorskip(
     "netCDF4", "1.6.2"
 )
 
+has_h5netcdf_1_7_0_or_above, requires_h5netcdf_1_7_0_or_above = _importorskip(
+    "h5netcdf", "1.7.0.dev"
+)
+
+has_netCDF4_1_7_0_or_above, requires_netCDF4_1_7_0_or_above = _importorskip(
+    "netCDF4", "1.7.0"
+)
+
 # change some global options for tests
 set_options(warn_for_unclosed_files=True)
 
@@ -170,8 +258,7 @@ class CountingScheduler:
         self.total_computes += 1
         if self.total_computes > self.max_computes:
             raise RuntimeError(
-                "Too many computes. Total: %d > max: %d."
-                % (self.total_computes, self.max_computes)
+                f"Too many computes. Total: {self.total_computes} > max: {self.max_computes}."
             )
         return dask.get(dsk, keys, **kwargs)
 
@@ -188,51 +275,6 @@ flaky = pytest.mark.flaky
 network = pytest.mark.network
 
 
-class UnexpectedDataAccess(Exception):
-    pass
-
-
-class InaccessibleArray(utils.NDArrayMixin, ExplicitlyIndexed):
-    """Disallows any loading."""
-
-    def __init__(self, array):
-        self.array = array
-
-    def get_duck_array(self):
-        raise UnexpectedDataAccess("Tried accessing data")
-
-    def __array__(self, dtype: np.typing.DTypeLike = None):
-        raise UnexpectedDataAccess("Tried accessing data")
-
-    def __getitem__(self, key):
-        raise UnexpectedDataAccess("Tried accessing data.")
-
-
-class FirstElementAccessibleArray(InaccessibleArray):
-    def __getitem__(self, key):
-        tuple_idxr = key.tuple
-        if len(tuple_idxr) > 1:
-            raise UnexpectedDataAccess("Tried accessing more than one element.")
-        return self.array[tuple_idxr]
-
-
-class DuckArrayWrapper(utils.NDArrayMixin):
-    """Array-like that prevents casting to array.
-    Modeled after cupy."""
-
-    def __init__(self, array: np.ndarray):
-        self.array = array
-
-    def __getitem__(self, key):
-        return type(self)(self.array[key])
-
-    def __array__(self, dtype: np.typing.DTypeLike = None):
-        raise UnexpectedDataAccess("Tried accessing data")
-
-    def __array_namespace__(self):
-        """Present to satisfy is_duck_array test."""
-
-
 class ReturnItem:
     def __getitem__(self, key):
         return key
@@ -270,9 +312,9 @@ def format_record(record) -> str:
 def assert_no_warnings():
     with warnings.catch_warnings(record=True) as record:
         yield record
-        assert (
-            len(record) == 0
-        ), f"Got {len(record)} unexpected warning(s): {[format_record(r) for r in record]}"
+        assert len(record) == 0, (
+            f"Got {len(record)} unexpected warning(s): {[format_record(r) for r in record]}"
+        )
 
 
 # Internal versions of xarray's test functions that validate additional
@@ -286,9 +328,45 @@ def assert_equal(a, b, check_default_indexes=True):
     xarray.testing._assert_internal_invariants(b, check_default_indexes)
 
 
-def assert_identical(a, b, check_default_indexes=True):
+def assert_identical(a, b, check_default_indexes=True, check_indexes=None):
+    """Assert that two xarray objects are identical.
+
+    This is a test-internal wrapper around xarray.testing.assert_identical
+    that also validates internal invariants.
+
+    Parameters
+    ----------
+    a, b : xarray objects
+        Objects to compare.
+    check_default_indexes : bool, default True
+        If True, validates that 1D dimension coordinates have default indexes
+        (internal invariant check). Set to False for objects that intentionally
+        lack default indexes.
+    check_indexes : bool, optional
+        If not specified (default), defaults to the value of check_default_indexes
+        for backwards compatibility.
+        If True (default), compare indexes as part of identity check.
+        If False, skip index comparison (only check data, attrs, names).
+    """
     __tracebackhide__ = True
-    xarray.testing.assert_identical(a, b)
+    # For backwards compatibility, check_default_indexes=False implies check_indexes=False
+    # unless check_indexes is explicitly specified
+    if check_indexes is None:
+        check_indexes = check_default_indexes
+    if check_indexes:
+        xarray.testing.assert_identical(a, b)
+    else:
+        # Drop all indexes before comparing to skip index comparison
+        from xarray import DataArray, Dataset
+
+        if isinstance(a, Dataset | DataArray):
+            a_no_idx = a.drop_indexes(list(a.xindexes))
+            b_no_idx = b.drop_indexes(list(b.xindexes))
+        else:
+            a_no_idx, b_no_idx = a, b
+
+        xarray.testing.assert_identical(a_no_idx, b_no_idx)
+
     xarray.testing._assert_internal_invariants(a, check_default_indexes)
     xarray.testing._assert_internal_invariants(b, check_default_indexes)
 
@@ -304,11 +382,12 @@ _DEFAULT_TEST_DIM_SIZES = (8, 9, 10)
 
 
 def create_test_data(
-    seed: int | None = None,
+    seed: int = 12345,
     add_attrs: bool = True,
     dim_sizes: tuple[int, int, int] = _DEFAULT_TEST_DIM_SIZES,
+    use_extension_array: bool = False,
 ) -> Dataset:
-    rs = np.random.RandomState(seed)
+    rs = np.random.default_rng(seed)
     _vars = {
         "var1": ["dim1", "dim2"],
         "var2": ["dim1", "dim2"],
@@ -320,33 +399,83 @@ def create_test_data(
     obj["dim2"] = ("dim2", 0.5 * np.arange(_dims["dim2"]))
     if _dims["dim3"] > 26:
         raise RuntimeError(
-            f'Not enough letters for filling this dimension size ({_dims["dim3"]})'
+            f"Not enough letters for filling this dimension size ({_dims['dim3']})"
         )
     obj["dim3"] = ("dim3", list(string.ascii_lowercase[0 : _dims["dim3"]]))
-    obj["time"] = ("time", pd.date_range("2000-01-01", periods=20))
+    obj["time"] = (
+        "time",
+        pd.date_range(
+            "2000-01-01",
+            periods=20,
+            unit="ns",
+        ),
+    )
     for v, dims in sorted(_vars.items()):
         data = rs.normal(size=tuple(_dims[d] for d in dims))
         obj[v] = (dims, data)
         if add_attrs:
             obj[v].attrs = {"foo": "variable"}
-
+    if use_extension_array:
+        obj["var4"] = (
+            "dim1",
+            pd.Categorical(
+                rs.choice(
+                    list(string.ascii_lowercase[: rs.integers(1, 5)]),
+                    size=dim_sizes[0],
+                )
+            ),
+        )
+        if has_pyarrow:
+            obj["var5"] = (
+                "dim1",
+                pd.array(
+                    rs.integers(1, 10, size=dim_sizes[0]).tolist(),
+                    dtype="int64[pyarrow]",
+                ),
+            )
     if dim_sizes == _DEFAULT_TEST_DIM_SIZES:
         numbers_values = np.array([0, 1, 2, 0, 0, 1, 1, 2, 2, 3], dtype="int64")
     else:
-        numbers_values = np.random.randint(0, 3, _dims["dim3"], dtype="int64")
+        numbers_values = rs.integers(0, 3, _dims["dim3"], dtype="int64")
     obj.coords["numbers"] = ("dim3", numbers_values)
     obj.encoding = {"foo": "bar"}
     assert_writeable(obj)
     return obj
 
 
-_CFTIME_CALENDARS = [
+_STANDARD_CALENDAR_NAMES = sorted(_STANDARD_CALENDARS_UNSORTED)
+_NON_STANDARD_CALENDAR_NAMES = {
+    "noleap",
     "365_day",
     "360_day",
     "julian",
     "all_leap",
     "366_day",
-    "gregorian",
-    "proleptic_gregorian",
-    "standard",
+}
+_NON_STANDARD_CALENDARS = [
+    pytest.param(cal, marks=requires_cftime)
+    for cal in sorted(_NON_STANDARD_CALENDAR_NAMES)
 ]
+_STANDARD_CALENDARS = [
+    pytest.param(cal, marks=requires_cftime if cal != "standard" else ())
+    for cal in _STANDARD_CALENDAR_NAMES
+]
+_ALL_CALENDARS = sorted(_STANDARD_CALENDARS + _NON_STANDARD_CALENDARS)
+_CFTIME_CALENDARS = [
+    pytest.param(*p.values, marks=requires_cftime) for p in _ALL_CALENDARS
+]
+
+
+def _all_cftime_date_types():
+    import cftime
+
+    return {
+        "noleap": cftime.DatetimeNoLeap,
+        "365_day": cftime.DatetimeNoLeap,
+        "360_day": cftime.Datetime360Day,
+        "julian": cftime.DatetimeJulian,
+        "all_leap": cftime.DatetimeAllLeap,
+        "366_day": cftime.DatetimeAllLeap,
+        "gregorian": cftime.DatetimeGregorian,
+        "proleptic_gregorian": cftime.DatetimeProlepticGregorian,
+    }
